MACHINE=
SCRIPT_NAME=elfm68hc12
OUTPUT_FORMAT="elf32-m68hc12"
TEXT_MEMORY=text
DATA_MEMORY=data
EEPROM_MEMORY=eeprom
ARCH=m68hc12
MAXPAGESIZE=32
GENERIC_BOARD=yes
TEMPLATE_NAME=elf32
EXTRA_EM_FILE=m68hc1xelf

