.*: Assembler messages:
.*:[0-9]+: Error: .\(%cr0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%cr7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%cr8\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%cr15\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%db0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%db7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%dr0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%dr7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%tr0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(%tr7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(cr0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(cr7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(cr8\). is not a valid base/index expression
.*:[0-9]+: Error: .\(cr15\). is not a valid base/index expression
.*:[0-9]+: Error: .\(db0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(db7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(dr0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(dr7\). is not a valid base/index expression
.*:[0-9]+: Error: .\(tr0\). is not a valid base/index expression
.*:[0-9]+: Error: .\(tr7\). is not a valid base/index expression
.*:[0-9]+: Error: .\[cr0\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[cr7\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[cr8\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[cr15\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[dr0\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[dr7\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[tr0\]. is not a valid base/index expression
.*:[0-9]+: Error: .\[tr7\]. is not a valid base/index expression
