SCRIPT_NAME=aout
TEXT_START_ADDR=0x2020
case ${LD_FLAG} in
    n|N)	TEXT_START_ADDR=0x2000 ;;
esac
OUTPUT_FORMAT="a.out-m68k-netbsd"
TARGET_PAGE_SIZE=0x2000
ARCH=m68k
EXECUTABLE_SYMBOLS='__DYNAMIC = 0;'
