[^:]*: Assembler messages:
[^:]*:1: Error: co-processor offset out of range
[^:]*:2: Error: co-processor offset out of range
[^:]*:3: Error: co-processor offset out of range
[^:]*:4: Error: co-processor offset out of range
[^:]*:5: Error: co-processor offset out of range
[^:]*:6: Error: co-processor offset out of range
