! Initially three separate sections, one with SHmedia and constants, one with
! SHcompact, and yet another with SHmedia.  Two .cranges sections
! generated by the assembler; two more needed at link time, as they will be
! consolidated into the same section, and mixed with a file with
! assembler-generated .cranges only and one without any .cranges.

	.section .text.shmedia,"ax"
	.mode SHmedia
	.align 2
sec1:
	nop
	nop
	nop
	nop
sec2:
	.long 41
	.long 43
	.long 42
	.long 43
	.long 42

	.section .text.shcompact,"ax"
	.align 1
	.mode SHcompact
sec3:
	nop
	nop
	nop

	.section .text.shmedia2,"ax"
	.align 2
sec4:
	.mode SHmedia
	nop
	nop
	nop
	nop
	nop
	nop
