 .macro try bytes:vararg
  .byte \bytes
  .byte 0x66, \bytes
  .byte 0x66, 0x48, \bytes
 .endm

 .text

_start:
	try	0x50
	try	0x58

	try	0x8f, 0xc0
	try	0x8f, 0x00

	try	0xff, 0xd0
	try	0xff, 0x10

	try	0xff, 0xe0
	try	0xff, 0x20

	try	0xff, 0xf0
	try	0xff, 0x30
