[^:]*: Assembler messages:
[^:]*:3: Error: r15 not allowed here -- `vld1.8 {d0},1f'
[^:]*:5: Error: r15 not allowed here -- `vld1.8 {D0},R0'
[^:]*:6: Error: r15 not allowed here -- `vld1.8 {Q1},R0'
[^:]*:7: Error: r15 not allowed here -- `vld1.8 {D0},\[PC\]'
[^:]*:8: Error: r15 not allowed here -- `vld1.8 {D0},\[PC,#0\]'
[^:]*:9: Error: r15 not allowed here -- `vst1.8 {D0},R0'
[^:]*:10: Error: r15 not allowed here -- `vst1.8 {Q1},R0'
[^:]*:11: Error: r15 not allowed here -- `vst1.8 {D0},\[PC\]'
[^:]*:12: Error: r15 not allowed here -- `vst1.8 {D0},\[PC,#0\]'
[^:]*:14: Error: r15 not allowed here -- `vld1.8 {d0},2f'
[^:]*:16: Error: r15 not allowed here -- `vld1.8 {D0},R0'
[^:]*:17: Error: r15 not allowed here -- `vld1.8 {Q1},R0'
[^:]*:18: Error: r15 not allowed here -- `vld1.8 {D0},\[PC\]'
[^:]*:19: Error: r15 not allowed here -- `vld1.8 {D0},\[PC,#0\]'
[^:]*:20: Error: r15 not allowed here -- `vst1.8 {D0},R0'
[^:]*:21: Error: r15 not allowed here -- `vst1.8 {Q1},R0'
[^:]*:22: Error: r15 not allowed here -- `vst1.8 {D0},\[PC\]'
[^:]*:23: Error: r15 not allowed here -- `vst1.8 {D0},\[PC,#0\]'
