	.level 1.1
	.code
	.align 4
; Basic add/sh?add instruction tests.  
;
; We could/should test some of the corner cases for register and 
; immediate fields.  We should also check the assorted field
; selectors to make sure they're handled correctly.
	add  %r4,%r5,%r6
	add,=  %r4,%r5,%r6
	add,<  %r4,%r5,%r6
	add,<=  %r4,%r5,%r6
	add,nuv  %r4,%r5,%r6
	add,znv  %r4,%r5,%r6
	add,sv  %r4,%r5,%r6
	add,od  %r4,%r5,%r6
	add,tr  %r4,%r5,%r6
	add,<>  %r4,%r5,%r6
	add,>=  %r4,%r5,%r6
	add,>  %r4,%r5,%r6
	add,uv  %r4,%r5,%r6
	add,vnz  %r4,%r5,%r6
	add,nsv  %r4,%r5,%r6
	add,ev  %r4,%r5,%r6

	addl  %r4,%r5,%r6
	addl,=  %r4,%r5,%r6
	addl,<  %r4,%r5,%r6
	addl,<=  %r4,%r5,%r6
	addl,nuv  %r4,%r5,%r6
	addl,znv  %r4,%r5,%r6
	addl,sv  %r4,%r5,%r6
	addl,od  %r4,%r5,%r6
	addl,tr  %r4,%r5,%r6
	addl,<>  %r4,%r5,%r6
	addl,>=  %r4,%r5,%r6
	addl,>  %r4,%r5,%r6
	addl,uv  %r4,%r5,%r6
	addl,vnz  %r4,%r5,%r6
	addl,nsv  %r4,%r5,%r6
	addl,ev  %r4,%r5,%r6

	addo  %r4,%r5,%r6
	addo,=  %r4,%r5,%r6
	addo,<  %r4,%r5,%r6
	addo,<=  %r4,%r5,%r6
	addo,nuv  %r4,%r5,%r6
	addo,znv  %r4,%r5,%r6
	addo,sv  %r4,%r5,%r6
	addo,od  %r4,%r5,%r6
	addo,tr  %r4,%r5,%r6
	addo,<>  %r4,%r5,%r6
	addo,>=  %r4,%r5,%r6
	addo,>  %r4,%r5,%r6
	addo,uv  %r4,%r5,%r6
	addo,vnz  %r4,%r5,%r6
	addo,nsv  %r4,%r5,%r6
	addo,ev  %r4,%r5,%r6

	addc  %r4,%r5,%r6
	addc,=  %r4,%r5,%r6
	addc,<  %r4,%r5,%r6
	addc,<=  %r4,%r5,%r6
	addc,nuv  %r4,%r5,%r6
	addc,znv  %r4,%r5,%r6
	addc,sv  %r4,%r5,%r6
	addc,od  %r4,%r5,%r6
	addc,tr  %r4,%r5,%r6
	addc,<>  %r4,%r5,%r6
	addc,>=  %r4,%r5,%r6
	addc,>  %r4,%r5,%r6
	addc,uv  %r4,%r5,%r6
	addc,vnz  %r4,%r5,%r6
	addc,nsv  %r4,%r5,%r6
	addc,ev  %r4,%r5,%r6

	addco  %r4,%r5,%r6
	addco,=  %r4,%r5,%r6
	addco,<  %r4,%r5,%r6
	addco,<=  %r4,%r5,%r6
	addco,nuv  %r4,%r5,%r6
	addco,znv  %r4,%r5,%r6
	addco,sv  %r4,%r5,%r6
	addco,od  %r4,%r5,%r6
	addco,tr  %r4,%r5,%r6
	addco,<>  %r4,%r5,%r6
	addco,>=  %r4,%r5,%r6
	addco,>  %r4,%r5,%r6
	addco,uv  %r4,%r5,%r6
	addco,vnz  %r4,%r5,%r6
	addco,nsv  %r4,%r5,%r6
	addco,ev  %r4,%r5,%r6

	add,l  %r4,%r5,%r6
	add,l,=  %r4,%r5,%r6
	add,l,<  %r4,%r5,%r6
	add,l,<=  %r4,%r5,%r6
	add,l,nuv  %r4,%r5,%r6
	add,l,znv  %r4,%r5,%r6
	add,l,sv  %r4,%r5,%r6
	add,l,od  %r4,%r5,%r6
	add,l,tr  %r4,%r5,%r6
	add,l,<>  %r4,%r5,%r6
	add,l,>=  %r4,%r5,%r6
	add,l,>  %r4,%r5,%r6
	add,l,uv  %r4,%r5,%r6
	add,l,vnz  %r4,%r5,%r6
	add,l,nsv  %r4,%r5,%r6
	add,l,ev  %r4,%r5,%r6

	add,tsv  %r4,%r5,%r6
	add,tsv,=  %r4,%r5,%r6
	add,tsv,<  %r4,%r5,%r6
	add,tsv,<=  %r4,%r5,%r6
	add,tsv,nuv  %r4,%r5,%r6
	add,tsv,znv  %r4,%r5,%r6
	add,tsv,sv  %r4,%r5,%r6
	add,tsv,od  %r4,%r5,%r6
	add,tsv,tr  %r4,%r5,%r6
	add,tsv,<>  %r4,%r5,%r6
	add,tsv,>=  %r4,%r5,%r6
	add,tsv,>  %r4,%r5,%r6
	add,tsv,uv  %r4,%r5,%r6
	add,tsv,vnz  %r4,%r5,%r6
	add,tsv,nsv  %r4,%r5,%r6
	add,tsv,ev  %r4,%r5,%r6

	add,c  %r4,%r5,%r6
	add,c,=  %r4,%r5,%r6
	add,c,<  %r4,%r5,%r6
	add,c,<=  %r4,%r5,%r6
	add,c,nuv  %r4,%r5,%r6
	add,c,znv  %r4,%r5,%r6
	add,c,sv  %r4,%r5,%r6
	add,c,od  %r4,%r5,%r6
	add,c,tr  %r4,%r5,%r6
	add,c,<>  %r4,%r5,%r6
	add,c,>=  %r4,%r5,%r6
	add,c,>  %r4,%r5,%r6
	add,c,uv  %r4,%r5,%r6
	add,c,vnz  %r4,%r5,%r6
	add,c,nsv  %r4,%r5,%r6
	add,c,ev  %r4,%r5,%r6

	add,c,tsv  %r4,%r5,%r6
	add,c,tsv,=  %r4,%r5,%r6
	add,c,tsv,<  %r4,%r5,%r6
	add,c,tsv,<=  %r4,%r5,%r6
	add,c,tsv,nuv  %r4,%r5,%r6
	add,c,tsv,znv  %r4,%r5,%r6
	add,c,tsv,sv  %r4,%r5,%r6
	add,c,tsv,od  %r4,%r5,%r6
	add,tsv,c,tr  %r4,%r5,%r6
	add,tsv,c,<>  %r4,%r5,%r6
	add,tsv,c,>=  %r4,%r5,%r6
	add,tsv,c,>  %r4,%r5,%r6
	add,tsv,c,uv  %r4,%r5,%r6
	add,tsv,c,vnz  %r4,%r5,%r6
	add,tsv,c,nsv  %r4,%r5,%r6
	add,tsv,c,ev  %r4,%r5,%r6
