.arm
.syntax unified
	fcpyd d3,d22
	fcpyd d22,d3
	fcvtds d22,s22
	fcvtsd s22,d22
	fmdhr d21,r4
	fmdlr d27,r5
	fmrdh r6,d23
	fmrdl r7,d25
	fsitod d22,s22
	fuitod d21,s21
	ftosid s20,d20
	ftosizd s20,d20
	ftouid s19,d19
	ftouizd s19,d19
	fldd d19,[r10,#4]
	fstd d21,[r10,#4]
	fldmiad r10!,{d5,d6}
	fldmiad r10!,{d18,d19,d20}
	fldmiax r10!,{d5,d6}
	fldmiax r10!,{d18,d19,d20}
	fldmdbx r10!,{d18,d19}
	fstmiad r9,{d20,d21,d22,d23,d24}
	fabsd d3,d5
	fabsd d12,d18
	fabsd d18,d19
	fnegd d3,d5
	fnegd d12,d18
	fnegd d18,d19
	fsqrtd d3,d5
	fsqrtd d12,d18
	fsqrtd d18,d19
	faddd d3,d5,d6
	faddd d12,d18,d4
	faddd d18,d19,d20
	fsubd d3,d5,d6
	fsubd d12,d18,d4
	fsubd d18,d19,d20
	fmuld d3,d5,d6
	fmuld d12,d18,d4
	fmuld d18,d19,d20
	fdivd d3,d5,d6
	fdivd d12,d18,d4
	fdivd d18,d19,d20
	fmacd d3,d5,d6
	fmacd d12,d18,d4
	fmacd d18,d19,d20
	fmscd d3,d5,d6
	fmscd d12,d18,d4
	fmscd d18,d19,d20
	fnmuld d3,d5,d6
	fnmuld d12,d18,d4
	fnmuld d18,d19,d20
	fnmacd d3,d5,d6
	fnmacd d12,d18,d4
	fnmacd d18,d19,d20
	fnmscd d3,d5,d6
	fnmscd d12,d18,d4
	fnmscd d18,d19,d20
	fcmpd d3,d18
	fcmpd d18,d3
	fcmpzd d19
	fcmped d3,d18
	fcmped d18,d3
	fcmpezd d19
	fmdrr d31,r3,r4
	fmrrd r5,r6,d30
