.*: Assembler messages:
.*:[[:digit:]]+: (Error|Warning): .* GPREL14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF64[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* PLTOFF14 .*
.*:[[:digit:]]+: (Error|Warning): .* PLTOFF32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* FPTR14 .*
.*:[[:digit:]]+: (Error|Warning): .* FPTR22 .*
.*:[[:digit:]]+: (Error|Warning): .* PCREL14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_FPTR14 .*
.*:[[:digit:]]+: (Error|Warning): .* SEGREL14 .*
.*:[[:digit:]]+: (Error|Warning): .* SEGREL22 .*
.*:[[:digit:]]+: (Error|Warning): .* SEGREL64I .*
.*:[[:digit:]]+: (Error|Warning): .* SECREL14 .*
.*:[[:digit:]]+: (Error|Warning): .* SECREL22 .*
.*:[[:digit:]]+: (Error|Warning): .* SECREL64I .*
.*:[[:digit:]]+: (Error|Warning): .* LTV14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTV22 .*
.*:[[:digit:]]+: (Error|Warning): .* LTV64I .*
.*:[[:digit:]]+: (Error|Warning): .* IPLT14 .*
.*:[[:digit:]]+: (Error|Warning): .* IPLT22 .*
.*:[[:digit:]]+: (Error|Warning): .* IPLT64I .*
.*:[[:digit:]]+: (Error|Warning): .* IPLT32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* IPLT64[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF14X .*
.*:[[:digit:]]+: (Error|Warning): .* TPREL32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_TPREL14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_TPREL64I .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_TPREL32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_TPREL64[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* DTPMOD14 .*
.*:[[:digit:]]+: (Error|Warning): .* DTPMOD22 .*
.*:[[:digit:]]+: (Error|Warning): .* DTPMOD64I .*
.*:[[:digit:]]+: (Error|Warning): .* DTPMOD32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_DTPMOD14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_DTPMOD64I .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_TPREL32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_TPREL64[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_DTPREL14 .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_DTPREL64I .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_DTPREL32[LM]SB .*
.*:[[:digit:]]+: (Error|Warning): .* LTOFF_DTPREL64[LM]SB .*
