	# cpu32 specific insns
	bgnd
	tblub	%d0,%d1,%d2
	tbluw	%d0,%d1,%d2
	tblul	%d0,%d1,%d2
	tblunb	%d0,%d1,%d2
	tblunw	%d0,%d1,%d2
	tblunl	%d0,%d1,%d2
	tblsb	%d0,%d1,%d2
	tblsw	%d0,%d1,%d2
	tblsl	%d0,%d1,%d2
	tblsnb	%d0,%d1,%d2
	tblsnw	%d0,%d1,%d2
	tblsnl	%d0,%d1,%d2
	tblub	(%a0),%d1
	tbluw	(%a0),%d1
	tblul	(%a0),%d1
	tblunb	(%a0),%d1
	tblunw	(%a0),%d1
	tblunl	(%a0),%d1
	tblsb	(%a0),%d1
	tblsw	(%a0),%d1
	tblsl	(%a0),%d1
	tblsnb	(%a0),%d1
	tblsnw	(%a0),%d1
	tblsnl	(%a0),%d1
