#source: attr-gnu-4-0.s
#source: attr-gnu-4-3.s
#as: -a32
#ld: -r -melf32ppc
#readelf: -A
#target: powerpc*-*-*

Attribute Section: gnu
File Attributes
  Tag_GNU_Power_ABI_FP: Single-precision hard float
