#readelf: -wl
#name: lns-duplicate
Raw dump of debug contents of section \.debug_line:
#...
 Line Number Statements:
  Extended opcode 2: set Address to .*
  Copy
  Set basic block
  .* by 1 to 2
#pass
