.*: Assembler messages:
.*:1: Error: AREGs are in bad order or same. Input text was A0.
.*:2: Error: AREGs are in bad order or same. Input text was A1.
.*:3: Error: AREGs are in bad order or same. Input text was A0.
.*:4: Error: AREGs are in bad order or same. Input text was a0.
.*:5: Error: AREGs are in bad order or same. Input text was a1.
.*:6: Error: AREGs are in bad order or same. Input text was a0.
.*:7: Error: AREGs are in bad order or same. Input text was a0.
.*:8: Error: AREGs are in bad order or same. Input text was a1.
.*:9: Error: AREGs are in bad order or same. Input text was a0.
.*:10: Error: Bad register in comparison. Input text was P0.
.*:11: Error: Bad register in comparison.
.*:12: Error: Bad register in comparison.
.*:13: Error: Bad register in comparison. Input text was R0.
.*:14: Error: Bad register in comparison.
.*:15: Error: Bad register in comparison.
.*:16: Error: Bad register in comparison.
.*:17: Error: Bad register in comparison. Input text was I0.
.*:18: Error: Bad register in comparison.
.*:19: Error: Bad register in comparison.
.*:20: Error: Bad register in comparison.
.*:21: Error: Bad register in comparison.
