.*: Assembler messages:
.*.s:[[:digit:]]+: Error: [Nn]umber of elements must be positive
.*.s:[[:digit:]]+: Error: [Nn]umber of elements must be positive
.*.s:[[:digit:]]+: Error: [Bb]ad or irreducible absolute expression
#pass
