[^:]*: Assembler messages:
[^:]*:3: Error: invalid barrier type -- `isb st'
[^:]*:4: Error: invalid barrier type -- `isb ish'
[^:]*:5: Error: invalid barrier type -- `isb ishst'
[^:]*:6: Error: invalid barrier type -- `isb nsh'
[^:]*:7: Error: invalid barrier type -- `isb nshst'
[^:]*:8: Error: invalid barrier type -- `isb osh'
[^:]*:9: Error: invalid barrier type -- `isb oshst'
