
Primary GOT:
 Canonical gp value: 000a7ff0

 Reserved entries:
   Address     Access  Initial Purpose
  000a0000 -32752\(gp\) 00000000 Lazy resolver
  000a0004 -32748\(gp\) 80000000 Module pointer \(GNU extension\)

 Local entries:
   Address     Access  Initial
  000a0008 -32744\(gp\) 000a0000

 Global entries:
   Address     Access  Initial Sym\.Val\. Type    Ndx Name
  000a000c -32740\(gp\) 00044030 00044030 FUNC    UND bar


PLT GOT:

 Reserved entries:
   Address  Initial Purpose
  00081000 00000000 PLT lazy resolver
  00081004 00000000 Module pointer

 Entries:
   Address  Initial Sym.Val. Type    Ndx Name
  00081008 00043020 00000000 FUNC    UND foo
