       .arch   armv6

foo:
       srsdb   r13, #13
       srsdb   r13!, #13
       srsia   r13, #13
       srsia   r13!, #13
       srsda   r13, #13
       srsda   r13!, #13
       srsib   r13, #13
       srsib   r13!, #13
       srsdb   r4, #13
       srsda   r4, #13
       srsia   r4, #13
       srsib   r4, #13

