
.*:     file format.*


Disassembly of section destsect:

00900000 <[^>]*>:
  900000:	e7fe      	b.n	900000 <dest>

Disassembly of section .text:

000080.. <[^>]*>:
    80..:	f0f7 9f.. 	b.w	900000 <dest>
