
.*:     file format .*


Disassembly of section \.text:

00000fe0 <_start>:
     fe0:	bf00      	nop
     fe2:	bf00      	nop
     fe4:	bf00      	nop
     fe6:	bf00      	nop
     fe8:	bf00      	nop
     fea:	bf00      	nop
     fec:	bf00      	nop
     fee:	bf00      	nop
     ff0:	bf00      	nop
     ff2:	bf00      	nop
     ff4:	bf00      	nop
     ff6:	bf00      	nop
     ff8:	bf00      	nop
     ffa:	ea81 0002 	eor.w	r0, r1, r2
     ffe:	f000 b80f 	b.w	1020 <_start\+0x40>
#...
    1020:	f7ff bfde 	b.w	fe0 <_start>
