# Two-operand variants of "R" and "I".
Main	FLOT X,Z
	FLOTU X,$233
	SFLOT $133,Z0
	FINT $123,Z
	SFLOT $133,Z
	FLOT X,Z0
	SFLOTU $123,Z0
	FIX $173,$233
	FLOTU $173,213
	FIXU X,Z
	FLOTU X,119
X IS $135
Z IS $244
Z0 IS 91
