#objdump: -dr

dump\.o:     file format .*


Disassembly of section \.text:

00000000 <\.text>:
   0:	7f 88                         	sstr\.b
   2:	7f 89                         	sstr\.w
   4:	7f 8a                         	sstr\.l
