#objdump: -dr

dump\.o:     file format .*


Disassembly of section \.text:

00000000 <\.text>:
   0:	7f 84                         	swhile\.b
   2:	7f 85                         	swhile\.w
   4:	7f 86                         	swhile\.l
