.*: Assembler messages:
.*:3: Error: operand out of domain \(-1 is not a multiple of 4\)
.*:4: Error: operand out of domain \(2 is not a multiple of 4\)
.*:5: Error: operand out of range.*
.*:6: Error: operand out of range.*
.*:7: Error: operand out of range.*
