.*: Assembler messages:
.*:7: Error: bad option.
.*:11: Error: bad option.
.*:12: Error: bad option.
.*:13: Error: bad option.
.*:14: Error: bad option.
.*:18: Error: bad option..
.*:22: Error: bad option.
.*:23: Error: bad option.
.*:24: Error: bad option.
.*:25: Error: bad option.
.*:26: Error: bad option.
.*:30: Error: bad option.
.*:31: Error: bad option.
.*:32: Error: bad option.
.*:33: Error: bad option.
.*:34: Error: bad option.
.*:35: Error: bad option.
.*:39: Error: bad option.
.*:43: Error: bad option.
.*:44: Error: bad option.
.*:45: Error: bad option.
.*:46: Error: bad option.
