#name: PE-COFF --export-dynamic warning
#target: *-*-mingw32 *-*-cygwin *-*-pe
#ld: --export-dynamic
#warning: warning: --export-dynamic is not supported for PE\+? targets, did you mean --export-all-symbols\?

