[^:]*: Assembler messages:
[^:]*:6: Error: 'cmtl' instruction not supported on this architecture
[^:]*:7: Error: 'll' instruction not supported on this architecture
[^:]*:8: Error: 'sl' instruction not supported on this architecture
[^:]*:14: Error: 'cmtl' instruction not supported on this architecture
[^:]*:15: Error: 'll' instruction not supported on this architecture
[^:]*:16: Error: 'sl' instruction not supported on this architecture
