[^:]*: Assembler messages:
[^:]*:3: Error: symbol var is in a different section
[^:]*:4: Error: undefined symbol undefinedvar used as an immediate value
[^:]*:5: Error: symbol var is in a different section
[^:]*:6: Error: undefined symbol undefinedvar used as an immediate value
