SCRIPT_NAME=aout
OUTPUT_FORMAT="a.out-sunos-big"
TEXT_START_ADDR=0x2020
case ${LD_FLAG} in
    n|N)	TEXT_START_ADDR=0x2000 ;;
esac
TARGET_PAGE_SIZE=0x2000
SEGMENT_SIZE=0x20000
ARCH=m68k
TEMPLATE_NAME=sunos
