# objdump: -h

.*:     file format elf64-mmix

Sections:
Idx Name          Size      VMA               LMA               File off  Algn
  0 \.text         0+4  0+  0+  0+40  2\*\*2
                  CONTENTS, ALLOC, LOAD, READONLY, CODE
  1 \.data         0+  0+  0+  0+44  2\*\*0
                  CONTENTS, ALLOC, LOAD, DATA
  2 \.bss          0+  0+  0+  0+44  2\*\*0
                  ALLOC
  3 \.MMIX\.reg_contents 0+6f0  0+  0+  0+44  2\*\*0
                  CONTENTS
