
.*:     file format.*


Disassembly of section destsect:

00009000 <[^>]*>:
    9000:	e7fe      	b.n	9000 <dest>

Disassembly of section .text:

000080.. <[^>]*>:
    80..:	f000 bf.. 	b.w	9000 <dest>
