SCRIPT_NAME=elf
OUTPUT_FORMAT="elf32-vax"
NO_REL_RELOCS=yes
TEXT_START_ADDR=0x10000
case ${LD_FLAG} in
    n|N)	TEXT_START_ADDR=0x1000 ;;
esac
MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
ARCH=vax
MACHINE=
NOP=0x0101
TEMPLATE_NAME=elf32
GENERATE_SHLIB_SCRIPT=yes
NO_SMALL_DATA=yes
