#nm: --extern-only
#name: MRI constants
#as: -M

# Test MRI style constants

0*0 T foo
0*a A s01
0*a A s02
0*a A s03
0*a A s04
0*a A s05
0*a A s06
0*a A s07
0*a A s08
0*a A s09
0*61 A s10
0*61 A s11
0*61626364 A s12
0*61276200 A s13
