[^:]*: Assembler messages:
[^:]*:[0-9]+: Error: r13 not allowed here -- `adc r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adc r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `adc r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adc r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `adc.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adc.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `adc.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adc.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `adc.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adc.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `add.w r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `addw r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `addw r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `add.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r15,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adds.w r15,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `add.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r0,r1,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r15,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adds.w r15,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `addw r15,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r15,r13,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `add.w r0,r13,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `add.w r0,r13,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `adr.w r13,test'
[^:]*:[0-9]+: Error: r15 not allowed here -- `adr.w r15,test'
[^:]*:[0-9]+: Error: r13 not allowed here -- `and r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `and r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `and r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `and r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `and.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `and.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `and.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `and.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `and.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `and.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `asr r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `asr r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `asr r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `asr r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `asr.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `asr.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `asr.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `asr.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `asr.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `asr.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bfc r13,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bfc r15,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bfi r13,r0,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bfi r15,r0,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bfi r0,r13,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bfi r0,r15,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bic r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bic r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bic r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bic r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bic.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bic.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bic.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bic.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bic.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bic.w r0,r1,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `blx r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `bxj r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `bxj r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `clz r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `clz r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `clz r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `clz r0,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmn r15,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmn.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `cmn.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmn.w r0,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmp.w r15,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmp r15,r0'
[^:]*:[0-9]+: Warning: use of r13 is deprecated
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmp r0,r15'
[^:]*:[0-9]+: Warning: use of r13 is deprecated
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmp.n r0,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmp.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `cmp.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `cmp.w r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `eor r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `eor r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `eor r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `eor r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `eor.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `eor.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `eor.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `eor.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `eor.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `eor.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsl r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsl r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsl r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsl r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsl.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsl.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsl.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsl.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsl.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsl.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsr r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsr r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsr r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsr r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsr.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsr.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsr.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsr.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `lsr.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `lsr.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mcr p0,#1,r13,cr0,cr0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mcr p0,#1,r15,cr0,cr0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mcrr p0,#1,r13,r0,cr0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mcrr p0,#1,r15,r0,cr0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mcrr p0,#1,r0,r13,cr0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mcrr p0,#1,r0,r15,cr0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mla r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mla r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mla r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mla r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mla r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mla r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mla r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mla r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mls r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mls r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mls r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mls r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mls r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mls r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mls r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mls r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mov.w r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mov.w r15,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mov.w r0,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mov.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `movs.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `movs.w r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `movs.w r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `movs.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mov.w r13,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mov.w r15,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mov.w r13,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mov.w r15,r15'
[^:]*:[0-9]+: Warning: Use of r13 as a source register is deprecated when r13 is the destination register.
[^:]*:[0-9]+: Warning: Use of r13 as a source register is deprecated when r15 is the destination register.
[^:]*:[0-9]+: Warning: Use of r15 as a source register is deprecated when r13 is the destination register.
[^:]*:[0-9]+: Warning: Use of r15 as a source register is deprecated when r15 is the destination register.
[^:]*:[0-9]+: Error: r13 not allowed here -- `movs r13,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `movs r15,r13'
[^:]*:[0-9]+: Error: r13 not allowed here -- `movs r13,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `movs r15,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `movt r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `movt r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mrc p0,#1,r13,cr0,cr0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mrrc p0,#1,r13,r0,cr0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mrrc p0,#1,r15,r0,cr0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mrrc p0,#1,r0,r13,cr0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mrrc p0,#1,r0,r15,cr0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mrs r13,cpsr'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mrs r15,cpsr'
[^:]*:[0-9]+: Error: r13 not allowed here -- `msr cpsr,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `msr cpsr,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mul r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mul r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mul r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mul r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mul r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mul r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mvn r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mvn r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mvn.w r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mvn.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `mvn.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `mvn.w r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orn r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orn r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orn r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orn r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orn r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orn r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orn r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orn r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orn r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orn r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orr r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orr r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orr r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orr r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orr r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orr r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orr r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orr r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `orr r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `orr r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `pkhbt r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `pkhbt r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `pkhbt r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `pkhbt r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `pkhbt r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `pkhbt r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `pld \[r0,r13\]'
[^:]*:[0-9]+: Error: r15 not allowed here -- `pld \[r0,r15\]'
[^:]*:[0-9]+: Error: cannot use register index with PC-relative addressing -- `pld \[r15,r0\]'
[^:]*:[0-9]+: Error: r13 not allowed here -- `pli \[r0,r13\]'
[^:]*:[0-9]+: Error: r15 not allowed here -- `pli \[r0,r15\]'
[^:]*:[0-9]+: Error: cannot use register index with PC-relative addressing -- `pli \[r15,r0\]'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qadd8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qadd8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qasx r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qasx r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qasx r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qasx r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qasx r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qasx r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qdadd r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qdadd r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qdadd r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qdadd r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qdadd r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qdadd r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qdsub r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qdsub r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qdsub r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qdsub r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qdsub r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qdsub r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsax r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsax r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsax r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsax r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsax r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsax r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `qsub8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `qsub8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rbit r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rbit r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rbit r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rbit r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rev.w r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rev.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rev.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rev.w r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rev16.w r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rev16.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rev16.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rev16.w r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `revsh.w r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `revsh.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `revsh.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `revsh.w r0,r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rfedb r15'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rfeia r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ror r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ror r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ror r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ror r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ror.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ror.w r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ror.w r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ror.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ror.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ror.w r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rrx r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rrx r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rrx r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rrx r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rsb.w r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rsb.w r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rsb.w r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rsb.w r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rsb r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rsb r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rsb r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rsb r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `rsb r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `rsb r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sadd16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sadd16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sadd16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sadd16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sadd16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sadd16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sadd8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sadd8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sadd8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sadd8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sadd8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sadd8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sasx r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sasx r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sasx r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sasx r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sasx r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sasx r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbc r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbc r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbc r0,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbc r0,r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbc r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbc r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbc r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbc r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbc r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbc r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbfx r13,r0,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbfx r15,r0,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sbfx r0,r13,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sbfx r0,r15,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sdiv r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sdiv r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sdiv r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sdiv r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sdiv r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sdiv r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sel r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sel r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sel r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sel r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sel r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sel r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shadd16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shadd16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shadd16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shadd16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shadd16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shadd16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shadd8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shadd8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shadd8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shadd8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shadd8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shadd8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shasx r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shasx r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shasx r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shasx r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shasx r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shasx r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsax r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsax r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsax r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsax r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsax r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsax r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsub16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsub16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsub16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsub16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsub16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsub16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsub8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsub8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsub8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsub8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `shsub8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `shsub8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlabb r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlabb r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlabb r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlabb r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlabb r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlabb r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlabb r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlabb r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlad r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlad r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlad r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlad r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlad r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlad r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlad r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlad r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlal r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlal r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlal r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlal r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlal r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlal r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlal r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlal r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlalbb r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlalbb r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlalbb r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlalbb r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlalbb r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlalbb r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlalbb r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlalbb r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlald r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlald r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlald r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlald r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlald r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlald r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlald r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlald r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlawb r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlawb r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlawb r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlawb r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlawb r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlawb r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlawb r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlawb r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsd r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsd r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsd r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsd r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsd r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsd r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsd r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsd r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsld r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsld r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsld r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsld r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsld r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsld r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smlsld r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smlsld r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmla r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmla r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmla r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmla r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmla r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmla r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmla r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmla r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmls r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmls r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmls r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmls r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmls r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmls r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmls r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmls r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmul r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmul r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmul r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmul r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smmul r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smmul r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smuad r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smuad r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smuad r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smuad r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smuad r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smuad r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smulbb r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smulbb r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smulbb r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smulbb r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smulbb r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smulbb r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smull r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smull r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smull r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smull r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smull r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smull r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smull r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smull r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smulwb r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smulwb r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smulwb r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smulwb r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smulwb r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smulwb r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smusd r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smusd r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smusd r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smusd r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `smusd r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `smusd r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssat r13,#1,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssat r15,#1,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssat r0,#1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssat r0,#1,r15'
[^:]*:[0-9]+: Error: shift expression is too large -- `ssat r1,#1,r3,asr#32'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssat16 r13,#1,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssat16 r15,#1,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssat16 r0,#1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssat16 r0,#1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssax r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssax r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssax r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssax r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssax r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssax r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssub16 r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssub16 r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssub16 r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssub16 r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssub16 r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssub16 r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssub8 r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssub8 r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssub8 r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssub8 r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ssub8 r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ssub8 r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sub.w r13,r0,#1'
[^:]*:[0-9]+: Error: only SUBS PC, LR, #const allowed -- `sub.w r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `subw r13,r0,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `subw r15,r0,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sub.w r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sub.w r15,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sub.w r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sub.w r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sub.w r0,r1,r15'
[^:]*:[0-9]+: Error: only SUBS PC, LR, #const allowed -- `sub.w r15,r13,#1'
[^:]*:[0-9]+: Error: only SUBS PC, LR, #const allowed -- `subs.w r15,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `subw r15,r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sub.w r15,r13,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sub.w r0,r13,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sub.w r0,r13,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtab r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtab r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtab r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtab r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtab r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtab r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtab16 r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtab16 r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtab16 r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtab16 r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtab16 r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtab16 r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtah r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtah r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtah r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtah r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtah r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtah r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtb r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtb r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtb r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtb r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtb16 r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtb16 r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxtb16 r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxtb16 r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxth r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxth r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `sxth r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `sxth r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tbb \[r13,r0]'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tbb \[r0,r13]'
[^:]*:[0-9]+: Error: r15 not allowed here -- `tbb \[r0,r15]'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tbh \[r13,r0]'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tbh \[r0,r13]'
[^:]*:[0-9]+: Error: r15 not allowed here -- `tbh \[r0,r15]'
[^:]*:[0-9]+: Error: r13 not allowed here -- `teq r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `teq r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `teq r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `teq r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `teq r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `teq r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tst r13,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `tst r15,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tst.w r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `tst.w r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `tst.w r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `tst.w r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uadd16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uadd16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uadd16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uadd16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uadd16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uadd16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uadd8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uadd8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uadd8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uadd8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uadd8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uadd8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uasx r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uasx r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uasx r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uasx r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uasx r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uasx r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ubfx r13,r0,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ubfx r15,r0,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `ubfx r0,r13,#1,#1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `ubfx r0,r15,#1,#1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `udiv r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `udiv r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `udiv r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `udiv r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `udiv r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `udiv r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhadd16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhadd16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhadd16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhadd16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhadd16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhadd16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhadd8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhadd8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhadd8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhadd8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhadd8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhadd8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhasx r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhasx r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhasx r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhasx r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhasx r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhasx r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsax r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsax r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsax r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsax r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsax r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsax r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsub16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsub16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsub16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsub16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsub16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsub16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsub8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsub8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsub8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsub8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uhsub8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uhsub8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umaal r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umaal r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umaal r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umaal r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umaal r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umaal r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umaal r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umaal r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umlal r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umlal r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umlal r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umlal r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umlal r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umlal r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umlal r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umlal r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umull r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umull r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umull r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umull r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umull r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umull r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `umull r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `umull r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqadd16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqadd16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqadd16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqadd16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqadd16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqadd16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqadd8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqadd8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqadd8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqadd8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqadd8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqadd8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqasx r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqasx r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqasx r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqasx r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqasx r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqasx r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsax r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsax r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsax r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsax r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsax r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsax r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsub16 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsub16 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsub16 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsub16 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsub16 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsub16 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsub8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsub8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsub8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsub8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uqsub8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uqsub8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usad8 r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usad8 r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usad8 r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usad8 r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usad8 r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usad8 r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usada8 r13,r0,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usada8 r15,r0,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usada8 r0,r13,r0,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usada8 r0,r15,r0,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usada8 r0,r0,r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usada8 r0,r0,r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usada8 r0,r0,r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usada8 r0,r0,r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usat r13,#1,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usat r15,#1,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usat r0,#1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usat r0,#1,r15'
[^:]*:[0-9]+: Error: shift expression is too large -- `usat r1,#1,r3,asr#32'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usat16 r13,#1,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usat16 r15,#1,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usat16 r0,#1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usat16 r0,#1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usax r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usax r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usax r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usax r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usax r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usax r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usub16 r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usub16 r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usub16 r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usub16 r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usub16 r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usub16 r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usub8 r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usub8 r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usub8 r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usub8 r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `usub8 r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `usub8 r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtab r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtab r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtab r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtab r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtab r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtab r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtab16 r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtab16 r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtab16 r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtab16 r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtab16 r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtab16 r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtah r13,r0,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtah r15,r0,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtah r0,r13,r1'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtah r0,r15,r1'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtah r0,r1,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtah r0,r1,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtb r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtb r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtb r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtb r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtb16 r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtb16 r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxtb16 r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxtb16 r0,r15'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxth r13,r0'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxth r15,r0'
[^:]*:[0-9]+: Error: r13 not allowed here -- `uxth r0,r13'
[^:]*:[0-9]+: Error: r15 not allowed here -- `uxth r0,r15'
