[^:]*: Assembler messages:
[^:]*:19: Error: PC-relative offset out of range
[^:]*:21: Error: PC-relative offset out of range
[^:]*:22: Error: PC-relative offset out of range
[^:]*:24: Error: PC-relative offset not 4-byte-aligned
[^:]*:53: Error: PC-relative offset out of range
[^:]*:55: Error: PC-relative offset out of range
[^:]*:56: Error: PC-relative offset out of range
[^:]*:58: Error: PC-relative offset not 4-byte-aligned
[^:]*:87: Error: PC-relative offset out of range
[^:]*:89: Error: PC-relative offset out of range
[^:]*:90: Error: PC-relative offset out of range
[^:]*:92: Error: PC-relative offset not 4-byte-aligned
[^:]*:121: Error: PC-relative offset out of range
[^:]*:123: Error: PC-relative offset out of range
[^:]*:124: Error: PC-relative offset out of range
[^:]*:126: Error: PC-relative offset not 4-byte-aligned
[^:]*:155: Error: PC-relative offset out of range
[^:]*:157: Error: PC-relative offset out of range
[^:]*:158: Error: PC-relative offset out of range
[^:]*:160: Error: PC-relative offset not 4-byte-aligned
[^:]*:189: Error: PC-relative offset out of range
[^:]*:191: Error: PC-relative offset out of range
[^:]*:192: Error: PC-relative offset out of range
[^:]*:194: Error: PC-relative offset not 4-byte-aligned
[^:]*:223: Error: PC-relative offset out of range
[^:]*:225: Error: PC-relative offset out of range
[^:]*:226: Error: PC-relative offset out of range
[^:]*:228: Error: PC-relative offset not 4-byte-aligned
[^:]*:257: Error: PC-relative offset out of range
[^:]*:259: Error: PC-relative offset out of range
[^:]*:260: Error: PC-relative offset out of range
[^:]*:262: Error: PC-relative offset not 4-byte-aligned
[^:]*:291: Error: PC-relative offset out of range
[^:]*:293: Error: PC-relative offset out of range
[^:]*:294: Error: PC-relative offset out of range
[^:]*:296: Error: PC-relative offset not 4-byte-aligned
[^:]*:325: Error: PC-relative offset out of range
[^:]*:327: Error: PC-relative offset out of range
[^:]*:328: Error: PC-relative offset out of range
[^:]*:330: Error: PC-relative offset not 4-byte-aligned
[^:]*:359: Error: PC-relative offset out of range
[^:]*:361: Error: PC-relative offset out of range
[^:]*:362: Error: PC-relative offset out of range
[^:]*:364: Error: PC-relative offset not 4-byte-aligned
