# Currently in the error messages the operand numbers for the constants
# aren't correct, which is why the patterns only check for ranges.
.*: Assembler messages:
.*:7: Error: Operand [345] of .alloc. should be .*
.*:8: Error: Operand [345] of .alloc. should be .*
.*:9: Error: Operand [345] of .alloc. should be .*
.*:10: Error: Operand [56] of .alloc. should be .*
.*:11: Error: Operand [234] of .alloc. should be .*
.*:12: Error: Operand [234] of .alloc. should be .*
.*:13: Error: Operand [234] of .alloc. should be .*
.*:14: Error: Operand [45] of .alloc. should be .*
