[^:]*: Assembler messages:
[^:]*:12: Error: SRS base register must be r13 -- `srsdb r4,#13'
[^:]*:13: Error: SRS base register must be r13 -- `srsda r4,#13'
[^:]*:14: Error: SRS base register must be r13 -- `srsia r4,#13'
[^:]*:15: Error: SRS base register must be r13 -- `srsib r4,#13'
