#readelf: -wl
#name: DWARF2 3
#not-target: ia64-*-* h8300-*-*

Raw dump of debug contents of section .debug_line:

  Offset:                      0x0
  Length:                      41
  DWARF Version:               2
  Prologue Length:             35
  Minimum Instruction Length:  [0-9]*
  Initial value of 'is_stmt':  1
  Line Base:                   -5
  Line Range:                  14
  Opcode Base:                 13

 Opcodes:
  Opcode 1 has 0 args
  Opcode 2 has 1 args
  Opcode 3 has 1 args
  Opcode 4 has 1 args
  Opcode 5 has 1 args
  Opcode 6 has 0 args
  Opcode 7 has 0 args
  Opcode 8 has 0 args
  Opcode 9 has 1 args
  Opcode 10 has 0 args
  Opcode 11 has 0 args
  Opcode 12 has 1 args

 The Directory Table is empty.

 The File Name Table:
  Entry	Dir	Time	Size	Name
  1	0	0	0	/beginwarn.c

 Line Number Statements:

