#source: start.s
#source: a.s
#source: pad2p26m32.s
#source: pad16.s
#source: pad4.s
#source: pad4.s
#source: jumpa.s
#as: -no-expand
#ld: -m elf64mmix
#objdump: -dr

.*:     file format elf64-mmix

Disassembly of section \.text:

0+ <_start>:
       0:	e3fd0001 	setl \$253,0x1

0+4 <a>:
       4:	e3fd0004 	setl \$253,0x4
	\.\.\.

0+4000000 <jumpa>:
 4000000:	e3fd0002 	setl \$253,0x2
 4000004:	f1000000 	jmp 4 <a>
 4000008:	e3fd0003 	setl \$253,0x3
