# A work in progress...
SCRIPT_NAME=elf
TEMPLATE_NAME=generic
EXTRA_EM_FILE=genelf
OUTPUT_FORMAT="elf32-i860-little"
BIG_OUTPUT_FORMAT="elf32-i860"
LITTLE_OUTPUT_FORMAT="elf32-i860-little"
NO_REL_RELOCS=yes
TEXT_START_ADDR=0
PAGE_SIZE=0x1000
MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
ARCH=i860
