	.text
	.am33
	call 256,[a2,a3,exreg0],9
	call 256,[a2,a3,exreg1],9
	call 256,[a2,a3,exother],9
	call 256,[a2,a3,all],9
	call 131071,[a2,a3,exreg0],9
	call 131071,[a2,a3,exreg1],9
	call 131071,[a2,a3,exother],9
	call 131071,[a2,a3,all],9
	movm (sp),[a2,a3,exreg0]
	movm (sp),[a2,a3,exreg1]
	movm (sp),[a2,a3,exother]
	movm (sp),[a2,a3,all]
	movm [a2,a3,exreg0],(sp)
	movm [a2,a3,exreg1],(sp)
	movm [a2,a3,exother],(sp)
	movm [a2,a3,all],(sp)
	movm (usp),[a2,a3,exreg0]
	movm (usp),[a2,a3,exreg1]
	movm (usp),[a2,a3,exother]
	movm (usp),[a2,a3,all]
	movm [a2,a3,exreg0],(usp)
	movm [a2,a3,exreg1],(usp)
	movm [a2,a3,exother],(usp)
	movm [a2,a3,all],(usp)
	mov usp,a0
	mov ssp,a1
	mov msp,a2
	mov pc,a3
	mov a0,usp
	mov a1,ssp
	mov a2,msp
	syscall 0x4
	mov epsw,d0
	mov d1,epsw
	mov a0,r1
	mov d2,r3
	mov r5,a1
	mov r7,d3
	mcst9 d0
	mcst48 d1
	getchx d0
	getclx d1
	and 131072,epsw
	or 65535,epsw
