
.*:     file format .*


Disassembly of section \.text:

00008f00 <targetfn>:
    8f00:	4770      	bx	lr
    8f02:	bf00      	nop
    8f04:	f3af 8000 	nop\.w

00008f08 <_start>:
    8f08:	bf00      	nop
    8f0a:	eb01 0002 	add\.w	r0, r1, r2
    8f0e:	f7ff bff7 	b\.w	8f00 <targetfn>
    8f12:	eb01 0002 	add\.w	r0, r1, r2
    8f16:	f7ff bff3 	b\.w	8f00 <targetfn>
    8f1a:	eb01 0002 	add\.w	r0, r1, r2
    8f1e:	f7ff bfef 	b\.w	8f00 <targetfn>
    8f22:	eb01 0002 	add\.w	r0, r1, r2
    8f26:	f7ff bfeb 	b\.w	8f00 <targetfn>
    8f2a:	eb01 0002 	add\.w	r0, r1, r2
    8f2e:	f7ff bfe7 	b\.w	8f00 <targetfn>
    8f32:	eb01 0002 	add\.w	r0, r1, r2
    8f36:	f7ff bfe3 	b\.w	8f00 <targetfn>
    8f3a:	eb01 0002 	add\.w	r0, r1, r2
    8f3e:	f7ff bfdf 	b\.w	8f00 <targetfn>
    8f42:	eb01 0002 	add\.w	r0, r1, r2
    8f46:	f7ff bfdb 	b\.w	8f00 <targetfn>
    8f4a:	eb01 0002 	add\.w	r0, r1, r2
    8f4e:	f7ff bfd7 	b\.w	8f00 <targetfn>
    8f52:	eb01 0002 	add\.w	r0, r1, r2
    8f56:	f7ff bfd3 	b\.w	8f00 <targetfn>
    8f5a:	eb01 0002 	add\.w	r0, r1, r2
    8f5e:	f7ff bfcf 	b\.w	8f00 <targetfn>
    8f62:	eb01 0002 	add\.w	r0, r1, r2
    8f66:	f7ff bfcb 	b\.w	8f00 <targetfn>
    8f6a:	eb01 0002 	add\.w	r0, r1, r2
    8f6e:	f7ff bfc7 	b\.w	8f00 <targetfn>
    8f72:	eb01 0002 	add\.w	r0, r1, r2
    8f76:	f7ff bfc3 	b\.w	8f00 <targetfn>
    8f7a:	eb01 0002 	add\.w	r0, r1, r2
    8f7e:	f7ff bfbf 	b\.w	8f00 <targetfn>
    8f82:	eb01 0002 	add\.w	r0, r1, r2
    8f86:	f7ff bfbb 	b\.w	8f00 <targetfn>
    8f8a:	eb01 0002 	add\.w	r0, r1, r2
    8f8e:	f7ff bfb7 	b\.w	8f00 <targetfn>
    8f92:	eb01 0002 	add\.w	r0, r1, r2
    8f96:	f7ff bfb3 	b\.w	8f00 <targetfn>
    8f9a:	eb01 0002 	add\.w	r0, r1, r2
    8f9e:	f7ff bfaf 	b\.w	8f00 <targetfn>
    8fa2:	eb01 0002 	add\.w	r0, r1, r2
    8fa6:	f7ff bfab 	b\.w	8f00 <targetfn>
    8faa:	eb01 0002 	add\.w	r0, r1, r2
    8fae:	f7ff bfa7 	b\.w	8f00 <targetfn>
    8fb2:	eb01 0002 	add\.w	r0, r1, r2
    8fb6:	f7ff bfa3 	b\.w	8f00 <targetfn>
    8fba:	eb01 0002 	add\.w	r0, r1, r2
    8fbe:	f7ff bf9f 	b\.w	8f00 <targetfn>
    8fc2:	eb01 0002 	add\.w	r0, r1, r2
    8fc6:	f7ff bf9b 	b\.w	8f00 <targetfn>
    8fca:	eb01 0002 	add\.w	r0, r1, r2
    8fce:	f7ff bf97 	b\.w	8f00 <targetfn>
    8fd2:	eb01 0002 	add\.w	r0, r1, r2
    8fd6:	f7ff bf93 	b\.w	8f00 <targetfn>
    8fda:	eb01 0002 	add\.w	r0, r1, r2
    8fde:	f7ff bf8f 	b\.w	8f00 <targetfn>
    8fe2:	eb01 0002 	add\.w	r0, r1, r2
    8fe6:	f7ff bf8b 	b\.w	8f00 <targetfn>
    8fea:	eb01 0002 	add\.w	r0, r1, r2
    8fee:	f7ff bf87 	b\.w	8f00 <targetfn>
    8ff2:	eb01 0002 	add\.w	r0, r1, r2
    8ff6:	f7ff bf83 	b\.w	8f00 <targetfn>
    8ffa:	eb01 0002 	add\.w	r0, r1, r2
    8ffe:	f000 b807 	b\.w	9010 <_start\+0x108>
    9002:	eb01 0002 	add\.w	r0, r1, r2
    9006:	f7ff bf7b 	b\.w	8f00 <targetfn>
    900a:	4770      	bx	lr
    900c:	f3af 8000 	nop\.w
    9010:	f7ff bf76 	b\.w	8f00 <targetfn>
