#source: attr-gnu-4-0.s
#source: attr-gnu-4-4.s
#as: -EB -32
#ld: -r -melf32btsmip
#readelf: -A
#target: mips*-*-*

Attribute Section: gnu
File Attributes
  Tag_GNU_MIPS_ABI_FP: Hard float \(MIPS32r2 64-bit FPU\)
