#source: loc8.s
#ld: -m mmo
#objdump: -str

# Setting file start through the special symbol, in-source, mmo version.

.*:     file format mmo
#...
Contents of section \.text:
 8000000000000000 f4000000                             .*
