	.hidden h1n1
	.hidden
	# next line has spaces
	.hidden                
	# next line has tabs
	.hidden			
	.internal i2
	.internal
	.protected porpoise
	.protected 
	.local foo
	.local
	.symver foo, foo@version1
	.symver
	.weak wimp
	.weak
	.type foo1,object
	.type 
