[^:]*: Assembler messages:
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcpyd d3,d22'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcpyd d22,d3'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcvtds d22,s22'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcvtsd s22,d22'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmdhr d21,r4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmdlr d27,r5'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmrdh r6,d23'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmrdl r7,d25'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fsitod d22,s22'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fuitod d21,s21'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `ftosid s20,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `ftosizd s20,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `ftouid s19,d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `ftouizd s19,d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fldd d19,\[r10,#4\]'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fstd d21,\[r10,#4\]'
[^:]*:[0-9]+: Error: register out of range in list -- `fldmiad r10!,\{d18,d19,d20\}'
[^:]*:[0-9]+: Error: register out of range in list -- `fldmiax r10!,\{d18,d19,d20\}'
[^:]*:[0-9]+: Error: register out of range in list -- `fldmdbx r10!,\{d18,d19\}'
[^:]*:[0-9]+: Error: register out of range in list -- `fstmiad r9,\{d20,d21,d22,d23,d24\}'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fabsd d12,d18'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fabsd d18,d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnegd d12,d18'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnegd d18,d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fsqrtd d12,d18'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fsqrtd d18,d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `faddd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `faddd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fsubd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fsubd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmuld d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmuld d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fdivd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fdivd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmacd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmacd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmscd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmscd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnmuld d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnmuld d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnmacd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnmacd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnmscd d12,d18,d4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fnmscd d18,d19,d20'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcmpd d3,d18'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcmpd d18,d3'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcmpzd d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcmped d3,d18'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcmped d18,d3'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fcmpezd d19'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmdrr d31,r3,r4'
[^:]*:[0-9]+: Error: D register out of range for selected VFP version -- `fmrrd r5,r6,d30'
