# Two-operand insns; 16-bit operand.
#
Main	SETH X,YZ
	SETMH $45,YZ
	ORH X,7892
	ORMH $45,12345

	SETML X,YZ
	SETL $45,YZ
	ORML X,7892
	ORL $45,12345

	INCH $45,12345
	INCMH X,7892
	ANDNH $45,YZ
	ANDNMH X,YZ

	INCML X,YZ
	INCL $45,YZ
	ANDNML X,7892
	ANDNL $45,12345
X IS $23
YZ IS #5678+456
