# 64bit SSSE3 New Instructions

	.text
foo:
	phaddw		(%rcx),%mm0
	phaddw		%mm1,%mm0
	phaddw		(%rcx),%xmm0
	phaddw		%xmm1,%xmm0
	phaddd		(%rcx),%mm0
	phaddd		%mm1,%mm0
	phaddd		(%rcx),%xmm0
	phaddd		%xmm1,%xmm0
	phaddsw		(%rcx),%mm0
	phaddsw		%mm1,%mm0
	phaddsw		(%rcx),%xmm0
	phaddsw		%xmm1,%xmm0
	phsubw		(%rcx),%mm0
	phsubw		%mm1,%mm0
	phsubw		(%rcx),%xmm0
	phsubw		%xmm1,%xmm0
	phsubd		(%rcx),%mm0
	phsubd		%mm1,%mm0
	phsubd		(%rcx),%xmm0
	phsubd		%xmm1,%xmm0
	phsubsw		(%rcx),%mm0
	phsubsw		%mm1,%mm0
	phsubsw		(%rcx),%xmm0
	phsubsw		%xmm1,%xmm0
	pmaddubsw	(%rcx),%mm0
	pmaddubsw	%mm1,%mm0
	pmaddubsw	(%rcx),%xmm0
	pmaddubsw	%xmm1,%xmm0
	pmulhrsw	(%rcx),%mm0
	pmulhrsw	%mm1,%mm0
	pmulhrsw	(%rcx),%xmm0
	pmulhrsw	%xmm1,%xmm0
	pshufb		(%rcx),%mm0
	pshufb		%mm1,%mm0
	pshufb		(%rcx),%xmm0
	pshufb		%xmm1,%xmm0
	psignb		(%rcx),%mm0
	psignb		%mm1,%mm0
	psignb		(%rcx),%xmm0
	psignb		%xmm1,%xmm0
	psignw		(%rcx),%mm0
	psignw		%mm1,%mm0
	psignw		(%rcx),%xmm0
	psignw		%xmm1,%xmm0
	psignd		(%rcx),%mm0
	psignd		%mm1,%mm0
	psignd		(%rcx),%xmm0
	psignd		%xmm1,%xmm0
	palignr		$0x2,(%rcx),%mm0
	palignr		$0x2,%mm1,%mm0
	palignr		$0x2,(%rcx),%xmm0
	palignr		$0x2,%xmm1,%xmm0
	pabsb		(%rcx),%mm0
	pabsb		%mm1,%mm0
	pabsb		(%rcx),%xmm0
	pabsb		%xmm1,%xmm0
	pabsw		(%rcx),%mm0
	pabsw		%mm1,%mm0
	pabsw		(%rcx),%xmm0
	pabsw		%xmm1,%xmm0
	pabsd		(%rcx),%mm0
	pabsd		%mm1,%mm0
	pabsd		(%rcx),%xmm0
	pabsd		%xmm1,%xmm0

	.p2align	4,0
