#as: -march=mips3 -mabi=64
#readelf: --relocs
#name: MIPS ELF reloc 21

Relocation section '\.rela\.data' .*:
.*
.* R_MIPS_GPREL32 * 0+00 * \.data \+ c
 * Type2: R_MIPS_NONE *
 * Type3: R_MIPS_NONE *
