[^:]*: Assembler messages:
[^:]*:2: Error: junk at end of line, first unrecognized character is `c'
[^:]*:3: Error: junk at end of line, first unrecognized character is `f'
[^:]*:6: Error: junk at end of line, first unrecognized character is `b'
[^:]*:7: Error: junk at end of line, first unrecognized character is `x'
