.*: Assembler messages:
.*:5: Warning: Macro instruction expanded into multiple instructions
.*:10: Warning: Macro instruction expanded into multiple instructions
.*:11: Warning: Macro instruction expanded into multiple instructions
.*:12: Warning: Macro instruction expanded into multiple instructions
.*:16: Warning: Macro instruction expanded into multiple instructions.*slot
.*:18: Warning: Macro instruction expanded into multiple instructions.*slot
.*:20: Warning: Macro instruction expanded into multiple instructions.*slot
