GAS for MMIX .*/list-pseudoints.s 			page 1


   1              	start
   2 0000 424344EE 	 tetra 0x424344ee
   3              	start2
   4 0004 00000000 	 octa  0xde0045007821329e
   4      DE004500 
   4      7821329E 
   5 0010 07D9     	 wyde 2009
   6 0012 99       	 byte 0x99
GAS for MMIX .*/list-pseudoints.s 			page 2


DEFINED SYMBOLS
.*/list-pseudoints.s:1      .text:0000000000000000 start
.*/list-pseudoints.s:3      .text:0000000000000004 start2

NO UNDEFINED SYMBOLS
