.*\.s: Assembler messages:
.*\.s:4: Error: Macro used \$at after ".set noat"
.*\.s:5: Error: Macro used \$at after ".set noat"
.*\.s:6: Error: Macro used \$at after ".set noat"
.*\.s:8: Warning: used \$at without ".set noat"
.*\.s:8: Warning: used \$at without ".set noat"
.*\.s:8: Warning: used \$at without ".set noat"
.*\.s:13: Warning: used \$26 with ".set at=\$26"
.*\.s:13: Warning: used \$26 with ".set at=\$26"
.*\.s:13: Warning: used \$26 with ".set at=\$26"
.*\.s:18: Warning: used \$27 with ".set at=\$27"
.*\.s:18: Warning: used \$27 with ".set at=\$27"
.*\.s:18: Warning: used \$27 with ".set at=\$27"
