[^:]*: Assembler messages:
[^:]*:11: Error: cannot use register index with PC-relative addressing -- `ldr r0,\[r1,pc,LSL#2\]'
[^:]*:12: Error: cannot use register index with PC-relative addressing -- `ldr r0,\[r1,pc,LSL#2\]!'
[^:]*:13: Error: cannot use register index with PC-relative addressing -- `ldr r0,\[r1\],pc,LSL#2'
[^:]*:14: Error: cannot use register index with PC-relative addressing -- `ldr r0,\[pc,r1,LSL#2\]!'
[^:]*:15: Error: cannot use register index with PC-relative addressing -- `ldr r0,\[pc\],r1,LSL#2'
[^:]*:18: Error: r15 not allowed here -- `ldrb pc,\[r0,#4\]'
[^:]*:19: Error: r15 not allowed here -- `ldrb pc,\[r0\],#4'
[^:]*:20: Error: r15 not allowed here -- `ldrb pc,\[r0,#4\]!'
[^:]*:23: Error: r15 not allowed here -- `ldrb pc,label'
[^:]*:24: Error: r15 not allowed here -- `ldrb pc,\[pc,#-0\]'
[^:]*:27: Error: r15 not allowed here -- `ldrb pc,\[r0,r1,LSL#2\]'
[^:]*:28: Error: r15 not allowed here -- `ldrb pc,\[r0,r1,LSL#2\]!'
[^:]*:29: Error: r15 not allowed here -- `ldrb pc,\[r0\],r1,LSL#2'
[^:]*:30: Error: cannot use register index with PC-relative addressing -- `ldrb r0,\[r1,pc,LSL#2\]'
[^:]*:31: Error: cannot use register index with PC-relative addressing -- `ldrb r0,\[r1,pc,LSL#2\]!'
[^:]*:32: Error: cannot use register index with PC-relative addressing -- `ldrb r0,\[r1\],pc,LSL#2'
[^:]*:33: Error: cannot use register index with PC-relative addressing -- `ldrb r0,\[pc,r1,LSL#2\]!'
[^:]*:34: Error: cannot use register index with PC-relative addressing -- `ldrb r0,\[pc\],r1,LSL#2'
[^:]*:37: Error: r15 not allowed here -- `ldrbt pc,\[r0\],#4'
[^:]*:38: Error: cannot use register index with PC-relative addressing -- `ldrbt r0,\[pc\],#4'
[^:]*:39: Error: r15 not allowed here -- `ldrbt pc,\[r0\],r1,LSL#4'
[^:]*:40: Error: cannot use register index with PC-relative addressing -- `ldrbt r0,\[pc\],r1,LSL#4'
[^:]*:41: Error: cannot use register index with PC-relative addressing -- `ldrbt r0,\[r1\],pc,LSL#4'
[^:]*:44: Error: r15 not allowed here -- `ldrd r0,pc,\[r1,#4\]'
[^:]*:45: Error: r15 not allowed here -- `ldrd r0,pc,\[r1\],#4'
[^:]*:46: Error: r15 not allowed here -- `ldrd r0,pc,\[r1,#4\]!'
[^:]*:49: Error: r15 not allowed here -- `ldrd r0,pc,label'
[^:]*:50: Error: r15 not allowed here -- `ldrd r0,pc,\[PC,#-0\]'
[^:]*:53: Error: r15 not allowed here -- `ldrd r0,pc,\[r1,r2\]'
[^:]*:54: Error: r15 not allowed here -- `ldrd r0,pc,\[r1,r2\]!'
[^:]*:55: Error: r15 not allowed here -- `ldrd r0,pc,\[r1\],r2'
[^:]*:56: Error: cannot use register index with PC-relative addressing -- `ldrd r0,r1,\[r2,pc\]'
[^:]*:57: Error: cannot use register index with PC-relative addressing -- `ldrd r0,r1,\[r2,pc\]!'
[^:]*:58: Error: cannot use register index with PC-relative addressing -- `ldrd r0,r1,\[r2\],pc'
[^:]*:59: Error: cannot use register index with PC-relative addressing -- `ldrd r0,r1,\[pc,r2\]!'
[^:]*:60: Error: cannot use register index with PC-relative addressing -- `ldrd r0,r1,\[pc\],r2'
[^:]*:63: Error: r15 not allowed here -- `ldrex pc,\[r0\]'
[^:]*:64: Error: instruction does not accept this addressing mode -- `ldrex r0,\[pc\]'
[^:]*:67: Error: r15 not allowed here -- `ldrexb pc,\[r0\]'
[^:]*:68: Error: r15 not allowed here -- `ldrexb r0,\[pc\]'
[^:]*:71: Error: r15 not allowed here -- `ldrexd r0,r1,\[pc\]'
[^:]*:74: Error: r15 not allowed here -- `ldrexh pc,\[r0\]'
[^:]*:75: Error: r15 not allowed here -- `ldrexh r0,\[pc\]'
[^:]*:78: Error: r15 not allowed here -- `ldrh pc,\[r0,#4\]'
[^:]*:79: Error: r15 not allowed here -- `ldrh pc,\[r0\],#4'
[^:]*:80: Error: r15 not allowed here -- `ldrh pc,\[r0,#4\]!'
[^:]*:83: Error: r15 not allowed here -- `ldrh pc,label'
[^:]*:84: Error: r15 not allowed here -- `ldrh pc,\[pc,#-0\]'
[^:]*:87: Error: r15 not allowed here -- `ldrh pc,\[r0,r1\]'
[^:]*:88: Error: r15 not allowed here -- `ldrh pc,\[r0,r1\]!'
[^:]*:89: Error: r15 not allowed here -- `ldrh pc,\[r0\],r1'
[^:]*:90: Error: cannot use register index with PC-relative addressing -- `ldrh r0,\[r1,pc\]'
[^:]*:91: Error: cannot use register index with PC-relative addressing -- `ldrh r0,\[r1,pc\]!'
[^:]*:92: Error: cannot use register index with PC-relative addressing -- `ldrh r0,\[r1\],pc'
[^:]*:93: Error: cannot use register index with PC-relative addressing -- `ldrh r0,\[pc,r1\]!'
[^:]*:94: Error: cannot use register index with PC-relative addressing -- `ldrh r0,\[pc\],r1'
[^:]*:97: Error: r15 not allowed here -- `ldrht pc,\[r0\],#4'
[^:]*:98: Error: cannot use writeback with PC-relative addressing -- `ldrht r0,\[pc\],#4'
[^:]*:99: Error: r15 not allowed here -- `ldrht pc,\[r0\],r1'
[^:]*:100: Error: cannot use register index with PC-relative addressing -- `ldrht r0,\[pc\],r1'
[^:]*:101: Error: cannot use register index with PC-relative addressing -- `ldrht r0,\[r1\],pc'
[^:]*:104: Error: r15 not allowed here -- `ldrsb pc,\[r0,#4\]'
[^:]*:105: Error: r15 not allowed here -- `ldrsb pc,\[r0\],#4'
[^:]*:106: Error: r15 not allowed here -- `ldrsb pc,\[r0,#4\]!'
[^:]*:109: Error: r15 not allowed here -- `ldrsb pc,label'
[^:]*:110: Error: r15 not allowed here -- `ldrsb pc,\[pc,#-0\]'
[^:]*:113: Error: r15 not allowed here -- `ldrsb pc,\[r0,r1\]'
[^:]*:114: Error: r15 not allowed here -- `ldrsb pc,\[r0,r1\]!'
[^:]*:115: Error: r15 not allowed here -- `ldrsb pc,\[r0\],r1'
[^:]*:116: Error: cannot use register index with PC-relative addressing -- `ldrsb r0,\[r1,pc\]'
[^:]*:117: Error: cannot use register index with PC-relative addressing -- `ldrsb r0,\[r1,pc\]!'
[^:]*:118: Error: cannot use register index with PC-relative addressing -- `ldrsb r0,\[r1\],pc'
[^:]*:119: Error: cannot use register index with PC-relative addressing -- `ldrsb r0,\[pc,r1\]!'
[^:]*:120: Error: cannot use register index with PC-relative addressing -- `ldrsb r0,\[pc\],r1'
[^:]*:123: Error: r15 not allowed here -- `ldrsbt pc,\[r0\],#4'
[^:]*:124: Error: cannot use writeback with PC-relative addressing -- `ldrsbt r0,\[pc\],#4'
[^:]*:125: Error: r15 not allowed here -- `ldrsbt pc,\[r0\],r1'
[^:]*:126: Error: cannot use register index with PC-relative addressing -- `ldrsbt r0,\[pc\],r1'
[^:]*:127: Error: cannot use register index with PC-relative addressing -- `ldrsbt r0,\[r1\],pc'
[^:]*:130: Error: r15 not allowed here -- `ldrsh pc,\[r0,#4\]'
[^:]*:131: Error: r15 not allowed here -- `ldrsh pc,\[r0\],#4'
[^:]*:132: Error: r15 not allowed here -- `ldrsh pc,\[r0,#4\]!'
[^:]*:135: Error: r15 not allowed here -- `ldrsh pc,label'
[^:]*:136: Error: r15 not allowed here -- `ldrsh pc,\[pc,#-0\]'
[^:]*:139: Error: r15 not allowed here -- `ldrsh pc,\[r0,r1\]'
[^:]*:140: Error: r15 not allowed here -- `ldrsh pc,\[r0,r1\]!'
[^:]*:141: Error: r15 not allowed here -- `ldrsh pc,\[r0\],r1'
[^:]*:142: Error: cannot use register index with PC-relative addressing -- `ldrsh r0,\[r1,pc\]'
[^:]*:143: Error: cannot use register index with PC-relative addressing -- `ldrsh r0,\[r1,pc\]!'
[^:]*:144: Error: cannot use register index with PC-relative addressing -- `ldrsh r0,\[r1\],pc'
[^:]*:145: Error: cannot use register index with PC-relative addressing -- `ldrsh r0,\[pc,r1\]!'
[^:]*:146: Error: cannot use register index with PC-relative addressing -- `ldrsh r0,\[pc\],r1'
[^:]*:149: Error: r15 not allowed here -- `ldrsht pc,\[r0\],#4'
[^:]*:150: Error: cannot use writeback with PC-relative addressing -- `ldrsht r0,\[pc\],#4'
[^:]*:151: Error: r15 not allowed here -- `ldrsht pc,\[r0\],r1'
[^:]*:152: Error: cannot use register index with PC-relative addressing -- `ldrsht r0,\[pc\],r1'
[^:]*:153: Error: cannot use register index with PC-relative addressing -- `ldrsht r0,\[r1\],pc'
[^:]*:156: Error: r15 not allowed here -- `ldrt pc,\[r0\],#4'
[^:]*:157: Error: cannot use register index with PC-relative addressing -- `ldrt r0,\[pc\],#4'
[^:]*:158: Error: r15 not allowed here -- `ldrt pc,\[r0\],r1,LSL#4'
[^:]*:159: Error: cannot use register index with PC-relative addressing -- `ldrt r0,\[pc\],r1,LSL#4'
[^:]*:160: Error: cannot use register index with PC-relative addressing -- `ldrt r0,\[r1\],pc,LSL#4'
[^:]*:166: Error: cannot use register index with PC-relative addressing -- `str r0,\[pc\],#4'
[^:]*:167: Error: cannot use register index with PC-relative addressing -- `str r0,\[pc,#4\]!'
[^:]*:170: Error: cannot use register index with PC-relative addressing -- `str r0,\[r1,pc,LSL#4\]'
[^:]*:171: Error: cannot use register index with PC-relative addressing -- `str r0,\[r1,pc,LSL#4\]!'
[^:]*:172: Error: cannot use register index with PC-relative addressing -- `str r0,\[r1\],pc,LSL#4'
[^:]*:175: Error: r15 not allowed here -- `strb pc,\[r0,#4\]'
[^:]*:176: Error: r15 not allowed here -- `strb pc,\[r0\],#4'
[^:]*:177: Error: r15 not allowed here -- `strb pc,\[r0,#4\]!'
[^:]*:178: Error: cannot use register index with PC-relative addressing -- `strb r0,\[pc\],#4'
[^:]*:179: Error: cannot use register index with PC-relative addressing -- `strb r0,\[pc,#4\]!'
[^:]*:182: Error: r15 not allowed here -- `strb pc,\[r0,r1,LSL#4\]'
[^:]*:183: Error: r15 not allowed here -- `strb pc,\[r0,r1,LSL#4\]!'
[^:]*:184: Error: r15 not allowed here -- `strb pc,\[r0\],r1,LSL#4'
[^:]*:185: Error: cannot use register index with PC-relative addressing -- `strb r1,\[r0,pc,LSL#4\]'
[^:]*:186: Error: cannot use register index with PC-relative addressing -- `strb r1,\[r0,pc,LSL#4\]!'
[^:]*:187: Error: cannot use register index with PC-relative addressing -- `strb r1,\[r0\],pc,LSL#4'
[^:]*:188: Error: cannot use register index with PC-relative addressing -- `strb r0,\[pc,r1,LSL#4\]!'
[^:]*:189: Error: cannot use register index with PC-relative addressing -- `strb r0,\[pc\],r1,LSL#4'
[^:]*:192: Error: r15 not allowed here -- `strbt pc,\[r0\],#4'
[^:]*:193: Error: cannot use register index with PC-relative addressing -- `strbt r0,\[pc\],#4'
[^:]*:194: Error: r15 not allowed here -- `strbt pc,\[r0\],r1,LSL#4'
[^:]*:195: Error: cannot use register index with PC-relative addressing -- `strbt r0,\[pc\],r1,LSL#4'
[^:]*:196: Error: cannot use register index with PC-relative addressing -- `strbt r0,\[r1\],pc,LSL#4'
[^:]*:199: Error: r15 not allowed here -- `strd r0,pc,\[r1,#4\]'
[^:]*:200: Error: r15 not allowed here -- `strd r0,pc,\[r1\],#4'
[^:]*:201: Error: r15 not allowed here -- `strd r0,pc,\[r1,#4\]!'
[^:]*:202: Error: cannot use writeback with PC-relative addressing -- `strd r0,r1,\[pc\],#4'
[^:]*:203: Error: cannot use writeback with PC-relative addressing -- `strd r0,r1,\[pc,#4\]!'
[^:]*:206: Error: r15 not allowed here -- `strd r0,pc,\[r1,r2\]'
[^:]*:207: Error: r15 not allowed here -- `strd r0,pc,\[r1,r2\]!'
[^:]*:208: Error: r15 not allowed here -- `strd r0,pc,\[r1\],r2'
[^:]*:209: Error: cannot use register index with PC-relative addressing -- `strd r0,r1,\[r2,pc\]'
[^:]*:210: Error: cannot use register index with PC-relative addressing -- `strd r0,r1,\[r2,pc\]!'
[^:]*:211: Error: cannot use register index with PC-relative addressing -- `strd r0,r1,\[r2\],pc'
[^:]*:212: Error: cannot use register index with PC-relative addressing -- `strd r0,r1,\[pc,r2\]!'
[^:]*:213: Error: cannot use register index with PC-relative addressing -- `strd r0,r1,\[pc\],r2'
[^:]*:216: Error: r15 not allowed here -- `strex pc,r0,\[r1\]'
[^:]*:217: Error: r15 not allowed here -- `strex r0,pc,\[r1\]'
[^:]*:218: Error: instruction does not accept this addressing mode -- `strex r0,r1,\[pc\]'
[^:]*:221: Error: r15 not allowed here -- `strexb pc,r0,\[r1\]'
[^:]*:222: Error: r15 not allowed here -- `strexb r0,pc,\[r1\]'
[^:]*:223: Error: instruction does not accept this addressing mode -- `strexb r0,r1,\[pc\]'
[^:]*:226: Error: r15 not allowed here -- `strexd pc,r0,r1,\[r2\]'
[^:]*:227: Error: r15 not allowed here -- `strexd r0,r1,r2,\[pc\]'
[^:]*:230: Error: r15 not allowed here -- `strexh pc,r0,\[r1\]'
[^:]*:231: Error: r15 not allowed here -- `strexh r0,pc,\[r1\]'
[^:]*:232: Error: instruction does not accept this addressing mode -- `strexh r0,r1,\[pc\]'
[^:]*:235: Error: r15 not allowed here -- `strh pc,\[r0,#4\]'
[^:]*:236: Error: r15 not allowed here -- `strh pc,\[r0\],#4'
[^:]*:237: Error: r15 not allowed here -- `strh pc,\[r0,#4\]!'
[^:]*:238: Error: cannot use writeback with PC-relative addressing -- `strh r0,\[pc\],#4'
[^:]*:239: Error: cannot use writeback with PC-relative addressing -- `strh r0,\[pc,#4\]!'
[^:]*:242: Error: r15 not allowed here -- `strh pc,\[r0,r1\]'
[^:]*:243: Error: r15 not allowed here -- `strh pc,\[r0,r1\]!'
[^:]*:244: Error: r15 not allowed here -- `strh pc,\[r0\],r1'
[^:]*:245: Error: cannot use register index with PC-relative addressing -- `strh r0,\[r1,pc\]'
[^:]*:246: Error: cannot use register index with PC-relative addressing -- `strh r0,\[r1,pc\]!'
[^:]*:247: Error: cannot use register index with PC-relative addressing -- `strh r0,\[r1\],pc'
[^:]*:248: Error: cannot use register index with PC-relative addressing -- `strh r0,\[pc,r1\]!'
[^:]*:249: Error: cannot use register index with PC-relative addressing -- `strh r0,\[pc\],r1'
[^:]*:252: Error: r15 not allowed here -- `strht pc,\[r0\],#4'
[^:]*:253: Error: cannot use writeback with PC-relative addressing -- `strht r0,\[pc\],#4'
[^:]*:254: Error: r15 not allowed here -- `strht pc,\[r0\],r1'
[^:]*:255: Error: cannot use register index with PC-relative addressing -- `strht r0,\[pc\],r1'
[^:]*:256: Error: cannot use register index with PC-relative addressing -- `strht r0,\[r1\],pc'
[^:]*:259: Error: cannot use register index with PC-relative addressing -- `strt r0,\[pc\],#4'
[^:]*:260: Error: cannot use register index with PC-relative addressing -- `strt r0,\[pc\],r1,LSL#4'
[^:]*:261: Error: cannot use register index with PC-relative addressing -- `strt r0,\[r1\],pc,LSL#4'
