[^:]*: Assembler messages:
[^:]*:7: Error: 'sploop' instruction not at start of execute packet
[^:]*:11: Error: 'sploopd' instruction not at start of execute packet
[^:]*:15: Error: 'sploopw' instruction not at start of execute packet
[^:]*:20: Error: 'spkernel' instruction not at start of execute packet
[^:]*:23: Error: 'spkernel' instruction not at start of execute packet
[^:]*:26: Error: 'spkernelr' instruction not at start of execute packet
[^:]*:28: Error: 'spmask' instruction not at start of execute packet
[^:]*:30: Error: 'spmaskr' instruction not at start of execute packet
