.explicit
.text
_start:
{.mfb
	nop.i	0
}
{.mlx
	nop.f	0
}
{.mfb
	nop.m	0
	nop.i	0
}
{.mfb
	nop.m	0
	nop.f	0
	nop.i	0
}
{.mlx
	nop.m	0
	nop.f	0
}
{.mfb
	nop	0
	nop	0
	nop	0
	nop	0
}
{.mlx
	nop	0
	nop	0
	nop	0
}
